@@ -6,7 +6,6 @@
 ##' @param db supported db, currently 'nuccore'
 ##' @param format one of 'genbank' or 'fasta'
 ##' @param outfile output file, by default, acc.gb or acc.fa
-##' @param ... additional parameters for download.file
 ##' @return output file vector
 ##' @importFrom magrittr %>%
 ##' @export
@@ -18,7 +17,7 @@
 ##' download_genbank(acc='AB115403', format='genbank', outfile=tmpgb)
 ##' download_genbank(acc='AB115403', format='fasta', outfile=tmpfa)
 ##' }
-download_genbank <- function(acc, db="nuccore", format = "genbank", outfile=NULL, ...) {
+download_genbank <- function(acc, db="nuccore", format = "genbank", outfile=NULL) {
     if (!is.null(outfile) && (length(outfile) != length(acc))) {
         stop("'outfile' & 'acc' should be equal length...")
     }
@@ -31,13 +30,13 @@ download_genbank <- function(acc, db="nuccore", format = "genbank", outfile=NULL
         } else {
             of <- outfile[i]
         }
-        ofs[i] <- download_genbank_item(acc[i], db, format, of, ...)
+        ofs[i] <- download_genbank_item(acc[i], db, format, of)
     }
     invisible(ofs)
 }
 
 ##' @importFrom utils download.file
-download_genbank_item <- function(acc, db="nuccore", format = "genbank", outfile=NULL, ...) {
+download_genbank_item <- function(acc, db="nuccore", format = "genbank", outfile=NULL) {
     format <- match.arg(format, c('genbank', 'fasta'))
     if (db != "nuccore") {
         stop("currently, only nuccore is supported...")
@@ -52,8 +51,50 @@ download_genbank_item <- function(acc, db="nuccore", format = "genbank", outfile
         outfile <- paste0(acc, suffix)
     }
     
-    downloader::download(url=url, destfile = outfile,  ...)
+    yulab.utils:::mydownload(url=url, destfile = outfile)
     invisible(outfile)
 }
 
+##' download fasta sequence by accession number and read as sequence object
+##'
+##'
+##' @title ncbi_fa_read
+##' @param acc accession number(s)
+##' @param db supported db, currently 'nuccore'
+##' @param type one of 'DNA', 'RNA', 'AA', 'unknown' or 'auto'
+##' @return BStringSet object
+##' @export
+##' @author Guangchuang Yu
+##' @examples
+##' \dontrun{
+##' x <- ncbi_fa_read(c('AB115403', 'AB115403.1'))
+##' x
+##' }
+ncbi_fa_read <- function(acc, db="nuccore", type = "auto") {
+    if (length(acc) == 0) {
+        stop("'acc' is empty...")
+    }
+    if (anyNA(acc)) {
+        stop("'acc' contains NA...")
+    }
+
+    if (db != "nuccore") {
+        stop("currently, only nuccore is supported...")
+    }
+
+    acc <- as.character(acc)
+    acc <- gsub("\\.\\d+", "", acc)
+
+    acc_query <- paste0(acc, collapse=",")
+    url <- paste0(
+        "https://www.ncbi.nlm.nih.gov/sviewer/viewer.cgi?tool=portal&sendto=on&log$=seqview&db=",
+        db,
+        "&dopt=fasta&sort=&val=",
+        acc_query
+    )
+
+    f <- tempfile(fileext = ".fasta")
+    yulab.utils:::mydownload(url=url, destfile=f)
+    fa_read(f, type=type)
+}
 
